#!/bin/bash
if [ $# -lt 3 ]
then
    echo "SYNTAX: $0 <prefix> <n-best> <testlist>"
    echo "EX: $0 nt5k/nt5k 10 ../data/g014a2.test.tabbed.bsf"
    exit 0
fi
prefix=${1}
nbest=${2}
testlist=${3}

echo "Generating nbest with modified ../phonetisaurus-g2p..."
echo "../phonetisaurus-g2p -m ${prefix}.fst -n ${nbest} -b ${nbest} -t ${testlist} | cut -f1,3- | cut -f1,2 > ${prefix}-${nbest}best-rnn.hyp"
echo ""
../phonetisaurus-g2p -m ${prefix}.fst -n ${nbest} -b ${nbest} -t ${testlist} | cut -f1,3- | cut -f1,2 > ${prefix}-${nbest}best-rnn.hyp

echo "Generating and evaluating 1-best with unmodified, repo version in g2p-nornnlm/phonetisaurus-g2p"
echo "./evaluate.py --modelfile ${prefix}.fst --testfile ${testlist} --prefix ${prefix}"
echo ""
./evaluate.py --modelfile ${prefix}.fst --testfile ${testlist} --prefix ${prefix}

echo "Reranking scores with rnnlm..."
echo "rnnlm-0.3c/rnnlm -test ${prefix}-${nbest}best-rnn.hyp -rnnlm ${prefix}.rnnlm -nbest -debug -0 > ${prefix}-${nbest}best-rnn.scores"
echo ""
rnnlm-0.3c/rnnlm -test ${prefix}-${nbest}best-rnn.hyp -rnnlm rnnlm-0.3c/${4} -nbest -debug -0 > ${prefix}-${nbest}best-rnn.scores

echo "Processing new 1-best based on rnnlm rescoring..."
echo "./process-nbest-for-rnnlm.py ${prefix}-${nbest}best-rnn.hyp ${prefix}-${nbest}best-rnn.scores > ${prefix}-${nbest}best-rnn-rescored-nbest.hyp"
./process-nbest-for-rnnlm.py ${prefix}-${nbest}best-rnn.hyp ${prefix}-${nbest}best-rnn.scores > ${prefix}-${nbest}best-rnn-rescored-nbest.hyp
echo "./process-nbest-for-rnnlm.py ${prefix}-${nbest}best-rnn-rescored-nbest.hyp > ${prefix}-rnn-1best-reranked.hyp"
echo ""
./process-nbest-for-rnnlm.py ${prefix}-${nbest}best-rnn-rescored-nbest.hyp > ${prefix}-rnn-1best-reranked.hyp

echo "Calculating new WER/PER scores based on re-ranking results..."
./calculateER.py -y ${prefix}-rnn-1best-reranked.hyp -r ${prefix}.ref
